@@ -2,6 +2,7 @@
  *      menu-cached.c
  *
  *      Copyright 2008 - 2010 PCMan <pcman.tw@gmail.com>
+ *      Copyright 2012-2013 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -61,6 +62,7 @@ typedef struct _Cache
     GFileMonitor** mons;
     /* GFileMonitor* cache_mon; */
 
+    gboolean need_reload;
     guint delayed_reload_handler;
     guint delayed_free_handler;
 
@@ -121,7 +123,7 @@ static void cache_free(Cache* cache)
 static gboolean read_all_used_files( FILE* f, int* n_files, char*** used_files )
 {
     char line[ 4096 ];
-    int i, n;
+    int i, n, x;
     char** files;
     int ver_maj, ver_min;
 
@@ -141,20 +143,31 @@ static gboolean read_all_used_files( FILE* f, int* n_files, char*** used_files )
     if( ! fgets( line, G_N_ELEMENTS(line), f ) )
         return FALSE;
 
-    *n_files = n = atoi( line );
+    n = atoi( line );
     files = g_new0( char*, n + 1 );
 
-    for( i = 0; i < n; ++i )
+    for( i = 0, x = 0; i < n; ++i )
     {
         int len;
+        GFile *gfile;
         if( ! fgets( line, G_N_ELEMENTS(line), f ) )
             return FALSE;
 
         len = strlen( line );
         if( len <= 1 )
             return FALSE;
-        files[ i ] = g_strndup( line, len - 1 ); /* don't include \n */
+        files[ x ] = g_strndup( line, len - 1 ); /* don't include \n */
+        gfile = g_file_new_for_path(files[x]+1);
+        if (g_file_query_exists(gfile, NULL))
+            x++;
+        else
+        {
+            DEBUG("ignoring not existant file from menu-cache-gen: %s", files[x]);
+            g_free(files[x]);
+        }
+        g_object_unref(gfile);
     }
+    *n_files = x;
     *used_files = files;
     return TRUE;
 }
@@ -229,7 +242,7 @@ static gboolean regenerate_cache( const char* menu_name,
     return TRUE;
 }
 
-static gboolean delayed_reload( Cache* cache )
+static void do_reload(Cache* cache)
 {
     GSList* l;
     char buf[38];
@@ -237,9 +250,6 @@ static gboolean delayed_reload( Cache* cache )
     int i;
     GFile* gf;
 
-    if(g_source_is_destroyed(g_main_current_source()))
-        return FALSE;
-
     /* DEBUG("Re-generation of cache is needed!"); */
     /* DEBUG("call menu-cache-gen to re-generate the cache"); */
     memcpy( buf, "REL:", 4 );
@@ -295,68 +305,29 @@ static gboolean delayed_reload( Cache* cache )
         if(write(g_io_channel_unix_get_fd(ch), buf, 37) < 37)
             g_io_channel_shutdown(ch, FALSE, NULL);
     }
-    cache->delayed_reload_handler = 0;
-    return FALSE;
+    cache->need_reload = FALSE;
 }
 
-static gboolean is_desktop_file_in_cache(Cache* cache, int dir_idx, const char* base_name)
+static gboolean delayed_reload( Cache* cache )
 {
-    gboolean ret = FALSE;
-    char* cache_file = g_build_filename(g_get_user_cache_dir(), "menus", cache->md5, NULL );
-    FILE* f = fopen(cache_file, "r");
-    g_free(cache_file);
-    if( f )
-    {
-        char line[4096];
-        while( fgets(line, G_N_ELEMENTS(line), f) )
-        {
-            int l = strlen(line);
-            line[l - 1] = '\0';
-            /* this is not a 100% safe way to check if the
-             * desktop file is listed in our original cache,
-             * but this works in 99.9% of cases. */
-            if( line[0] == '-' && 0 == strcmp(line+1, base_name) )
-            {
-                int i;
-                /* if this line is -desktop_id, we can get its dir_idx in following 5 lines. */
-                for( i = 0; i < 5; ++i )
-                {
-                    if( ! fgets(line, G_N_ELEMENTS(line), f) )
-                        break;
-                    if( i == 3 && !line[0] ) /* the 4th line should be empty */
-                        break;
-                }
-                if( i >= 5 )
-                {
-                    if( dir_idx == atoi(line) )
-                    {
-                        ret = TRUE;
-                        break;
-                    }
-                }
-            }
-            else if(strcmp(line, base_name) == 0)
-            {
-                /* if this line is a basename, we can get its dir_idx in the next line. */
-                if( fgets( line, G_N_ELEMENTS(line), f) )
-                {
-                    if( dir_idx == atoi(line) )
-                    {
-                        ret = TRUE;
-                        break;
-                    }
-                }
-            }
-        }
-        fclose(f);
-    }
-    return ret;
+    if(g_source_is_destroyed(g_main_current_source()))
+        return FALSE;
+
+    cache->delayed_reload_handler = 0;
+    if(cache->need_reload)
+        do_reload(cache);
+
+    return FALSE;
 }
 
 void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
                       GFileMonitorEvent evt, Cache* cache )
 {
-    /* DEBUG("file %s is changed (%d).", g_file_get_path(gf), evt); */
+#ifdef G_ENABLE_DEBUG
+    char *path = g_file_get_path(gf);
+    g_debug("file %s is changed (%d).", path, evt);
+    g_free(path);
+#endif
     /* if( mon != cache->cache_mon ) */
     {
         /* Optimization: Some files in the dir are changed, but it
@@ -384,29 +355,6 @@ void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
                 /* only *.desktop and *.directory files can affect the content of the menu. */
                 if( g_str_has_suffix(base_name, ".desktop") )
                 {
-                    /* FIXME: there seems to be some problems here... so weird. */
-                    /* further optimization:
-                     * If the changed file is a desktop entry file with
-                     * Hidden=true or NoDisplay=true, and it's not in our
-                     * original menu cache, either, just omit it since it won't
-                     * affect the content of our menu.
-                     */
-                    gboolean in_cache = is_desktop_file_in_cache(cache, idx, base_name);
-                    /* DEBUG("in_cache = %d", in_cache); */
-                    if( ! in_cache ) /* means this is a new file or previously hidden */
-                    {
-                        GKeyFile* kf = g_key_file_new();
-                        if( g_key_file_load_from_file( kf, changed_file, 0, NULL ) )
-                        {
-                            if( ! g_key_file_get_boolean(kf, "Desktop Entry", "Hidden", NULL)
-                                && ! g_key_file_get_boolean(kf, "Desktop Entry", "NoDisplay", NULL) )
-                            {
-                                skip = FALSE;
-                            }
-                        }
-                        g_key_file_free(kf);
-                    }
-                    else
                         skip = FALSE;
                 }
                 else if( g_str_has_suffix(base_name, ".directory") )
@@ -414,35 +362,6 @@ void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
 
                 if( skip )
                 {
-                    /* FIXME: utime to update the mtime of cached file. */
-                    /* FIXME: temporarily disable monitor of the cached file before utime() */
-                    /* without this, directory mtime will > mtime of cache file,
-                     * and the menu will get re-generated unnecessarily the next time. */
-                    struct utimbuf ut;
-                    char* cache_file;
-
-                    g_free(changed_file);
-
-                    cache_file = g_build_filename(g_get_user_cache_dir(), "menus", cache->md5, NULL );
-
-                    ut.actime = ut.modtime = time(NULL);
-                    /* stop monitor of cache file. */
-                    /*
-                    g_file_monitor_cancel( cache->cache_mon );
-                    g_object_unref( cache->cache_mon );
-                    cache->cache_mon = NULL;
-                    */
-                    /* update the mtime of the cache file. */
-                    utime( cache_file, &ut );
-
-                    /* restart the monitor */
-                    /* gf = g_file_new_for_path( cache_file ); */
-                    g_free(cache_file);
-                    /*
-                    cache->cache_mon = g_file_monitor_file(gf, 0, NULL, NULL);
-                    g_object_unref(gf);
-                    g_signal_connect( cache->cache_mon, "changed", G_CALLBACK(on_file_changed), cache);
-                    */
                     DEBUG("files are changed, but no re-generation is needed.");
                     return;
                 }
@@ -452,7 +371,14 @@ void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
     }
 
     if( cache->delayed_reload_handler )
+    {
+        /* we got some change in last 3 seconds... not reload again */
+        cache->need_reload = TRUE;
         g_source_remove(cache->delayed_reload_handler);
+    }
+    else
+        /* no reload in last 3 seconds... good, do it immediately */
+        do_reload(cache);
 
     cache->delayed_reload_handler = g_timeout_add_seconds_full( G_PRIORITY_LOW, 3, (GSourceFunc)delayed_reload, cache, NULL );
 }
@@ -579,7 +505,6 @@ static void on_client_closed(gpointer user_data)
         }
     }
     /* DEBUG("client closed"); */
-    g_io_channel_unref(ch);
 }
 
 static gboolean on_client_data_in(GIOChannel* ch, GIOCondition cond, gpointer user_data)
@@ -607,7 +532,7 @@ retry:
     --len;
     line[len] = 0;
 
-    DEBUG("line(%d) = %s", len, line);
+    DEBUG("line(%d) = %s", (int)len, line);
 
     if( memcmp(line, "REG:", 4) == 0 )
     {
@@ -760,8 +685,8 @@ static gboolean on_new_conn_incoming(GIOChannel* ch, GIOCondition cond, gpointer
 static void terminate(int sig)
 {
 /* #ifndef HAVE_ABSTRACT_SOCKETS */
-    char path[256];
-    get_socket_name(path, 256);
+    char path[1024];
+    get_socket_name(path, sizeof(path));
     unlink(path);
     exit(0);
 /* #endif */
@@ -771,7 +696,6 @@ static gboolean on_server_conn_close(GIOChannel* ch, GIOCondition cond, gpointer
 {
     /* FIXME: is this possible? */
     /* the server socket is accidentally closed. terminate the server. */
-    g_io_channel_unref(ch);
     terminate(SIGTERM);
     return TRUE;
 }
@@ -803,6 +727,9 @@ int main(int argc, char** argv)
         exit(0);
     }
 
+    /* reset session to forget about parent process completely */
+    setsid();
+
     /* change working directory to root, so previous working directory
      * can be unmounted */
     if (chdir("/") < 0) {
@@ -847,7 +774,9 @@ int main(int argc, char** argv)
     g_io_add_watch(ch, G_IO_ERR, on_server_conn_close, NULL);
     g_io_channel_unref(ch);
 
+#if !GLIB_CHECK_VERSION(2, 36, 0)
     g_type_init();
+#endif
 
     hash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, NULL);
 
